//==============================================================
//
// Bcl_canvas
//
//==============================================================
#ifndef _BCL_CANVAS_H_
#define _BCL_CANVAS_H_

#include "bcl_typedef.h"
#include "bcl_bmp.h"
#include "bcl_font.h"

class Bcl_imglist;
class Bcl_wnd;

/// `NX
class Bcl_canvas
{
	Bcl_wnd* m_parent;
	HWND m_handle;
	COLORREF m_bg_color;
	COLORREF m_color;
	COLORREF m_text_color;
	BRUSH_TYPE m_brush;
	PEN_TYPE m_pen;
	int m_left;
	int m_right;
	int m_top;
	int m_bottom;
	HDC m_hdc;
	HBITMAP		m_canvas;		//zEBhEłrbg}bv
	bool m_focus;
	POINT m_start;
	Bcl_font m_font;
	
	HBRUSH create_brush(COLORREF col, BRUSH_TYPE brush);
	int get_pen_type();
public:
	Bcl_canvas();
	Bcl_canvas(Bcl_wnd* parent, int x = GetSystemMetrics(SM_CXFULLSCREEN), int y = GetSystemMetrics(SM_CYFULLSCREEN));
	virtual ~Bcl_canvas();
	bool set_canvas(Bcl_wnd* parent, int x = GetSystemMetrics(SM_CXFULLSCREEN), int y = GetSystemMetrics(SM_CYFULLSCREEN));
	void set_bg_color(COLORREF col);
	void set_color(COLORREF col);
	void set_text_color(COLORREF col);
	void set_brush_type(BRUSH_TYPE brush);
	void set_pen_type(PEN_TYPE pen);
	void clear();
	void dot(int x, int y);
	void line(int sx, int sy, int ex, int ey);
	void box(int sx, int sy, int ex, int ey);
	void circle(int x, int y, int r);
	void ellipse(int sx, int sy, int ex, int ey);
	void arc(int sx, int sy, int ex, int ey, int srx, int sry, int erx, int ery);
	void chord(int sx, int sy, int ex, int ey, int srx, int sry, int erx, int ery);
	void polygon(const std::vector<POINT>& pts);
	void redraw(int sx, int sy, int ex, int ey);
	void drag_rect(int x, int y, bool flag);
	void paint();
	Bcl_bmp* get_bmp(int x, int y, int w, int h) const;
	void put_bmp(const Bcl_bmp& bmp, int x, int y, DWORD ro);
	void fit_bmp(const Bcl_bmp& bmp, int x, int y, int w, int h, DWORD ro, int mode = BLACKONWHITE);
	void print_text(int x, int y, const Bcl_str& str, int w = 0, UINT fmt = DT_LEFT | DT_NOCLIP);
	Bcl_str get_modify_text_by_print(const Bcl_str& str, int w = 0, UINT fmt = DT_LEFT | DT_NOCLIP);
	void print_htext(int x, int y, const Bcl_str& str, int size, UINT fmt = DT_LEFT | DT_NOCLIP);
	void show_imglist(const Bcl_imglist& list, int x, int y, int num);
	void show_all_imglist(const Bcl_imglist& list, int x, int y);
	COLORREF get_pixel(int x, int y) const;
	void set_font(const Bcl_font& font);
	void set_font_size(int size);
	void set_font_height(int height);
	void set_font_angle(int angle);
	void set_font_effect(LONG bold, bool italic, bool underline, bool strikeout);
	const Bcl_font& get_font() const;
	int get_font_height() const;
};
#endif
