//==============================================================
//
// Bcl_printer
//
//==============================================================
#ifndef _BCL_PRINTER_H_
#define _BCL_PRINTER_H_

#include "bcl_typedef.h"
#include "bcl_str.h"
#include "bcl_font.h"
#include "bcl_bmp.h"
#include "bcl_canvas.h"

class Bcl_font;
class Bcl_wnd;

enum PAPER_SIZE
{
	PAPER_A4_VERT,
	PAPER_A4_HORZ,
	PAPER_B5_VERT,
	PAPER_B5_HORZ,
};

/// v^\
struct Prn_info
{
	Bcl_str name;
	Bcl_str port_name;
};

/// v^NX
class Bcl_printer
{
	HDC m_hdc;
	COLORREF m_color;
	COLORREF m_text_color;
	BRUSH_TYPE m_brush;
	PEN_TYPE m_pen;
	Bcl_font m_font;
	PAPER_SIZE m_paper_size;
	Bcl_str m_name;
	DOCINFO m_docinfo;
	std::vector<Prn_info> m_prn_info;
	Bcl_canvas m_preview_cvs;
	int m_width;
	int m_height;
	int m_mergin_up;
	int m_mergin_down;
	int m_mergin_left;
	int m_mergin_right;
	bool m_is_preview;

	HBRUSH create_brush(COLORREF col, BRUSH_TYPE type);
	int get_pen_type();
public:
	Bcl_printer();
	~Bcl_printer();
	Bcl_str get_default_printer(Bcl_wnd* parent) const;
	int get_printer_num() const;
	Bcl_str get_printer_name(int idx) const;
	Bcl_str get_port_name(int idx) const;
	void set_paper_size(PAPER_SIZE size);
	void set_document_name(const Bcl_str& name);
	void create_dc(int idx);
	void delete_dc();
	void print_start();
	void print_end();
	void preview_start(Bcl_wnd* parent);
	void preview_end();
	Bcl_bmp* get_preview_bmp() const;
	void next_page();
	PAPER_SIZE get_paper_size() const;
	int get_width() const;
	int get_height() const;
	void set_color(COLORREF col);
	void set_text_color(COLORREF col);
	void set_brush_type(BRUSH_TYPE brush);
	void set_pen_type(PEN_TYPE pen);
	void dot(int x, int y);
	void line(int sx, int sy, int ex, int ey);
	void box(int sx, int sy, int ex, int ey);
	void circle(int x, int y, int r);
	void ellipse(int sx, int sy, int ex, int ey);
	void arc(int sx, int sy, int ex, int ey, int srx, int sry, int erx, int ery);
	void chord(int sx, int sy, int ex, int ey, int srx, int sry, int erx, int ery);
	void polygon(const std::vector<POINT>& pts);
	void put_bmp(const Bcl_bmp& bmp, int x, int y, DWORD ro);
	void fit_bmp(const Bcl_bmp& bmp, int x, int y, int w, int h, DWORD ro, int mode = BLACKONWHITE);
	void print_text(int x, int y, const Bcl_str& str, int w = 0, UINT fmt = DT_LEFT | DT_NOCLIP);
	void print_htext(int x, int y, const Bcl_str& str, int size, UINT fmt = DT_LEFT | DT_NOCLIP);
	void set_font(const Bcl_font& font);
	void set_font_size(int size);
	void set_font_height(int height);
	void set_font_angle(int angle);
	void set_font_effect(LONG bold, bool italic, bool underline, bool strikeout);
	const Bcl_font& get_font() const;
	int get_font_height(int size) const;
};

#endif
