//==============================================================
//
// Bcl_wnd
//
//==============================================================
#ifndef _BCL_WND_H_
#define _BCL_WND_H_

#include "bcl_typedef.h"
#include "bcl_str.h"
#include "bcl_id.h"

class Bcl_wnd;
class Bcl_dlg;
class Bcl_redit;
class Bcl_sdi;
class Bcl_mdi;
class Bcl_mdichild;

/// bZ[W\
struct Message
{
	Message(UINT m, UINT i, void (Bcl_wnd::*f)(WPARAM, LPARAM)) : msg(m), id(i), func(f){}
	UINT msg;
	int id;
	void (Bcl_wnd::*func)(WPARAM, LPARAM);
};

typedef std::vector<Message> MessageMap;

/// EBhENX
class Bcl_wnd
{
	friend class Bcl_dlg;
	friend class Bcl_redit;
	friend class Bcl_sdi;
	friend class Bcl_mdi;
	friend class Bcl_mdichild;

	Bcl_id		m_menu;
	Bcl_id		m_icon;
	Bcl_id		m_cursor;
	HBRUSH		m_brush;

	void set_handle(HWND handle);
	void set_instance(HINSTANCE instance);
protected:
	Bcl_str		m_class_name;		// NX
	HWND		m_handle;			// EBhEnh
	MessageMap	m_msg;				// bZ[W}bv
	HINSTANCE	m_instance;			// CX^X
	WNDPROC		m_proc;				// 
	Bcl_str 	m_window_name;		// EBhE
	DWORD		m_style;			// EBhEX^C
	DWORD		m_ex_style;			// EBhEgX^C
	int			m_id;				// Rg[hc
	LRESULT		m_proc_rtn;			// EBhEvVW̖߂l
	std::vector<HWND> m_dlg_handle;

	static std::map<HWND, Bcl_wnd*> m_wnd_list;		// EBhEXg
	static Bcl_wnd* m_cur_wnd;						// ݂̃EBhE

	MessageMap* get_message_map();

	virtual void on_create(WPARAM wp, LPARAM lp);
	virtual void on_close(WPARAM wp, LPARAM lp);
	bool create_wnd(Bcl_wnd* parent, int x = CW_USEDEFAULT, int y = CW_USEDEFAULT, int w = CW_USEDEFAULT, int h = CW_USEDEFAULT);
	bool create(HWND parent = NULL, int x = CW_USEDEFAULT, int y = CW_USEDEFAULT, int w = CW_USEDEFAULT, int h = CW_USEDEFAULT);

	static LRESULT CALLBACK wnd_proc(HWND handle, UINT msg, WPARAM wp, LPARAM lp);
	static bool proc_msg(HWND handle, UINT msg, WPARAM wp, LPARAM lp);

public:
	Bcl_wnd();
	virtual ~Bcl_wnd();
	void set_menu(const Bcl_str& menu);
	void set_menu(int menu);
	void set_icon(const Bcl_str& icon);
	void set_icon(int icon);
	void set_cursor(const Bcl_str& cursor);
	void set_cursor(int cursor);
	void set_cursor(LPCTSTR cursor);
	void set_brush(HBRUSH brush);
	void set_window_name(LPCTSTR window_name);
	void set_style(DWORD style);
	DWORD get_style() const;
	void set_expand_style(DWORD ex_style);
	bool init(const Bcl_str& class_name, HINSTANCE instance, bool is_enable_dblclk = false);
	HWND get_handle() const;
	HINSTANCE get_instance() const;
	LRESULT send_msg(UINT msg, WPARAM wp, LPARAM lp);
	LRESULT send_item_msg(int id, UINT msg, WPARAM wp, LPARAM lp);
	LRESULT post_msg(UINT msg, WPARAM wp, LPARAM lp);
	void set_text(const Bcl_str& text);
	Bcl_str get_text() const;
	void set_enabled(bool flg);
	void show();
	void hide();
	void set_capture();
	void release_capture();
	void set_active_window();
	void set_focus();
	void set_foreground_window();
	void invalidate(bool erase);
	void invalidate_rect(int x, int y, int w, int h, bool erase);
	void set_clipboard_data(const Bcl_str& data);
	void set_proc_rtn(LRESULT rtn);
	virtual void move(int x, int y, int w, int h);
	int get_height() const;
	int get_width() const;
	RECT get_rect() const;
	int get_client_height() const;
	int get_client_width() const;
	RECT get_client_rect() const;
	POINT get_screen_from_client(POINT pt) const;
	POINT get_client_from_screen(POINT pt) const;
	int show_msg_box(const Bcl_str& text, const Bcl_str& caption, UINT type);
	long get_window_long(int idx) const;
	long set_window_long(int idx, long val);
};
#endif
