//==============================================================
//
// Bcl_font
//
//==============================================================
#include "bcl_font.h"
#include "bcl_wnd.h"
#include "bcl_str.h"

/// RXgN^
Bcl_font::Bcl_font() : m_font(NULL)
{
	m_lf.lfWidth = 0;
	m_lf.lfWeight =FW_REGULAR;
	m_lf.lfItalic = FALSE;
	m_lf.lfUnderline = FALSE;
	m_lf.lfStrikeOut = FALSE;
	m_lf.lfCharSet = SHIFTJIS_CHARSET;
	m_lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
	m_lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	m_lf.lfQuality = PROOF_QUALITY;
	m_lf.lfPitchAndFamily = FIXED_PITCH | FF_ROMAN;
}

/// RXgN^
/// @param font tHgnh
Bcl_font::Bcl_font(HFONT font) : m_font(NULL)
{
	set_handle(font);
}

/// RXgN^
/// @param parent eEBhE
/// @param size tHgTCY
/// @param name tHg
/// @param angle px
Bcl_font::Bcl_font(Bcl_wnd* parent, int size, const Bcl_str& name, int angle) : m_font(NULL)
{
	m_lf.lfWidth = 0;
	m_lf.lfWeight =FW_REGULAR;
	m_lf.lfItalic = FALSE;
	m_lf.lfUnderline = FALSE;
	m_lf.lfStrikeOut = FALSE;
	m_lf.lfCharSet = SHIFTJIS_CHARSET;
	m_lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
	m_lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	m_lf.lfQuality = PROOF_QUALITY;
	m_lf.lfPitchAndFamily = FIXED_PITCH | FF_ROMAN;
	set_font(parent, size, name, angle);
}

/// RXgN^
/// @param height tHg
/// @param name tHg
/// @param angle px
Bcl_font::Bcl_font(int height, const Bcl_str& name, int angle) : m_font(NULL)
{
	m_lf.lfWidth = 0;
	m_lf.lfWeight =FW_REGULAR;
	m_lf.lfItalic = FALSE;
	m_lf.lfUnderline = FALSE;
	m_lf.lfStrikeOut = FALSE;
	m_lf.lfCharSet = SHIFTJIS_CHARSET;
	m_lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
	m_lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	m_lf.lfQuality = PROOF_QUALITY;
	m_lf.lfPitchAndFamily = FIXED_PITCH | FF_ROMAN;
	set_font(height, name, angle);
}

/// Rs[RXgN^
/// @param ref Rs[
Bcl_font::Bcl_font(const Bcl_font& ref) : m_font(NULL)
{
	copy(ref);
}

/// fXgN^
Bcl_font::~Bcl_font()
{
	if (m_font != NULL)
	{
		DeleteObject(m_font);
	}
}

/// Zq
/// @param ref Rs[
/// @return g̎Q
Bcl_font& Bcl_font::operator=(const Bcl_font& ref)
{
	copy(ref);
	return *this;
}

/// Rs[
/// @param ref Rs[
void Bcl_font::copy(const Bcl_font& ref)
{
	if (m_font != NULL)
	{
		DeleteObject(m_font);
	}
	m_font = NULL;
	if (ref.m_font != NULL)
	{
		memcpy(&m_lf, &ref.m_lf, sizeof(LOGFONT));
		m_font = CreateFontIndirect(&m_lf);
	}
}

/// tHgݒ
/// @param parent eEBhE
/// @param size tHgTCY
/// @param name tHg
/// @param angle px
void Bcl_font::set_font(Bcl_wnd* parent, int size, const Bcl_str& name, int angle)
{
	HWND wnd = parent->get_handle();
	HDC hdc = GetDC(wnd);
	set_font(MulDiv(size, GetDeviceCaps(hdc, LOGPIXELSY), -72), name, angle);
	ReleaseDC(wnd, hdc);
}

/// tHgݒ
/// @param height tHg
/// @param name tHg
/// @param angle px
void Bcl_font::set_font(int height, const Bcl_str& name, int angle)
{
	//ɃtHgݒ肪ĂΏ
	if (m_font != NULL)
	{
		DeleteObject(m_font);
	}
	m_lf.lfHeight = height;
	m_lf.lfEscapement = angle;
	m_lf.lfOrientation = angle;
	strcpy(m_lf.lfFaceName, name.to_char());
	m_font = CreateFontIndirect(&m_lf);
}

/// tHgTCYݒ
/// @param parent eEBhE
/// @param size tHgTCY
void Bcl_font::set_size(Bcl_wnd* parent, int size)
{
	if (m_font == NULL)
	{
		return;
	}
	set_font(parent, size, m_lf.lfFaceName, m_lf.lfEscapement);
}

/// tHgݒ
/// @param height tHg
void Bcl_font::set_height(int height)
{
	if (m_font == NULL)
	{
		return;
	}
	set_font(height, m_lf.lfFaceName, m_lf.lfEscapement);
}

/// pxݒ
/// @param angle px
void Bcl_font::set_angle(int angle)
{
	if (m_font == NULL)
	{
		return;
	}
	set_font(m_lf.lfHeight, m_lf.lfFaceName, angle);
}

/// px擾
/// @return px
int Bcl_font::get_angle() const
{
	return m_lf.lfEscapement;
}

/// L`FbN
/// @return L
bool Bcl_font::is_valid() const
{
	if (m_font == NULL)
	{
		return false;
	}
	return true;
}

/// tHgnh擾
/// @return tHgnh
HFONT Bcl_font::get_handle() const
{
	return m_font;
}

/// tHgnhݒ
/// @param font tHgnh
void Bcl_font::set_handle(HFONT font)
{
	if (m_font != NULL)
	{
		DeleteObject(m_font);
	}
	GetObject((HGDIOBJ)font, sizeof(LOGFONT), &m_lf);
	m_font = CreateFontIndirect(&m_lf);
}

/// tHg擾
/// @return tHg
int Bcl_font::get_height() const
{
	return abs(m_lf.lfHeight);

}

/// tHǧʂݒ肵܂
/// @param bold 
/// @param italic Α
/// @param underline 
/// @param strikeout 
void Bcl_font::set_font_effect(LONG bold, bool italic, bool underline, bool strikeout)
{
	m_lf.lfWeight = bold;
	m_lf.lfItalic = italic;
	m_lf.lfUnderline = underline;
	m_lf.lfStrikeOut = strikeout;
	if (m_font != NULL)
	{
		set_font(m_lf.lfHeight, m_lf.lfFaceName, m_lf.lfEscapement);
	}
}

/// 擾
/// @return 
LONG Bcl_font::get_bold() const
{
	return m_lf.lfWeight;
}

/// Α̎
/// @return 
bool Bcl_font::is_italic() const
{
	return m_lf.lfItalic;
}

/// A_[C
/// @return 
bool Bcl_font::is_underline() const
{
	return m_lf.lfUnderline;
}

/// 
/// @return 
bool Bcl_font::is_strikeout() const
{
	return m_lf.lfStrikeOut;
}

/// |CgTCY擾
/// @return |CgTCY
int Bcl_font::get_point_size(Bcl_wnd* parent) const
{
	HWND wnd = parent->get_handle();
	HDC hdc = GetDC(wnd);
	int size = MulDiv(m_lf.lfHeight, -72, GetDeviceCaps(hdc, LOGPIXELSY));
	ReleaseDC(wnd, hdc);
	return size;
}

/// LN^Zbgݒ肵܂
/// @param charset LN^Zbg
void Bcl_font::set_charset(BYTE charset)
{
	m_lf.lfCharSet = charset;
	if (m_font != NULL)
	{
		set_font(m_lf.lfHeight, m_lf.lfFaceName, m_lf.lfEscapement);
	}
}

/// LN^Zbg擾
/// @return LN^Zbg
BYTE Bcl_font::get_charset() const
{
	return m_lf.lfCharSet;
}

/// sb`t@~[ݒ肵܂
/// @param pitch_and_family sb`t@~[
void Bcl_font::set_pitch_and_family(BYTE pitch_and_family)
{
	m_lf.lfPitchAndFamily = pitch_and_family;
	if (m_font != NULL)
	{
		set_font(m_lf.lfHeight, m_lf.lfFaceName, m_lf.lfEscapement);
	}
}

/// sb`t@~[擾
/// @return sb`t@~[
BYTE Bcl_font::get_pitch_and_family() const
{
	return m_lf.lfPitchAndFamily;
}

/// tHg̎擾
/// @return tHg
Bcl_str Bcl_font::get_font_name() const
{
	return m_lf.lfFaceName;
}
