#include "bcl_itr.h"
#include "bcl_layout_horz.h"

/// RXgN^
Bcl_layout_horz::Bcl_layout_horz()
{
	Layout_horz_info info;
	info.layout = NULL;
	info.is_flex_width = false;
	info.is_flex_height = false;
	m_info.push_back(info);
	m_flex_num = 0;
	m_pt.x = 0x7fffffff;
	m_pt.y = 0x7fffffff;
	m_width = 0;
	m_height = 0;
}

/// fXgN^
Bcl_layout_horz::~Bcl_layout_horz()
{
	Bcl_itr< std::vector<Layout_horz_info> > itr(m_info);
	while (itr.has_next())
	{
		Layout_horz_info& info = itr.next();
		if (info.layout != NULL) delete info.layout;
	}
}

/// TCYݒ
/// @param x xW
/// @param y yW
/// @param w 
/// @param h 
void Bcl_layout_horz::set_initial_size(int x, int y, int w, int h)
{
	m_pt.x = x;
	m_pt.y = y;
	m_height = h;
	m_width = w;
	Layout_horz_info* info_prev = NULL;
	Bcl_itr< std::vector<Layout_horz_info> > itr(m_info);
	while (itr.has_next())
	{
		Layout_horz_info& info = itr.next();
		if (info.layout != NULL)
		{
			POINT pt = info.layout->get_position();
			info_prev->width = pt.x - x;
			x += info_prev->width;
			info.width = info.layout->get_width();
			info.layout->set_initial_size(x, y, info.width, h);
			info.height = info.layout->get_height();
			info.y = info.layout->get_position().y - y;
			x += info.width;
		}
		info_prev = &info;
	}
	info_prev->width = w - x;
}

/// ړ
/// @param mover mover
/// @param x xW
/// @param y yW
/// @param w 
/// @param h 
void Bcl_layout_horz::move(Bcl_mover& mover, int x, int y, int w, int h)
{
	int div = 0;
	int div_mod = 0;
	int mod_num = 0;
	if (m_flex_num > 0)
	{
		div = (w - m_width) / m_flex_num;
		if (w > m_width)
		{
			mod_num = (w - m_width) % m_flex_num;
			if (mod_num > 0) div_mod = 1;
		}
		else
		{
			mod_num = (m_width - w) % m_flex_num;
			if (mod_num > 0) div_mod = -1;
		}
	}
	Bcl_itr< std::vector<Layout_horz_info> > itr(m_info);
	while (itr.has_next())
	{
		Layout_horz_info& info = itr.next();
		int width = info.width;
		if (info.is_flex_width)
		{
			width += div + div_mod;
			if (mod_num > 0)
			{
				mod_num--;
				div_mod = 0;
			}
		}
		int height = info.height;
		if (info.is_flex_height) height += h - m_height;
		if (info.layout != NULL) info.layout->move(mover, x, y + info.y, width, height);
		x += width;
	}
}

/// ʒu擾܂
/// @return ʒu
POINT Bcl_layout_horz::get_position() const
{
	return m_pt;
}

/// 擾܂
/// @return 
int Bcl_layout_horz::get_width() const
{
	return m_width;
}

/// 擾܂
/// @return 
int Bcl_layout_horz::get_height() const
{
	return m_height;
}

/// CAEgԂɃolǉ܂
void Bcl_layout_horz::add_spring()
{
	int idx = m_info.size() - 1;
	if (!m_info[idx].is_flex_width)
	{
		m_info[idx].is_flex_width = true;
		m_flex_num++;
	}
}

/// CAEgǉ܂
/// @param layout CAEg
/// @param is_flex_width ςɂ邩
/// @param is_flex_height ςɂ邩
void Bcl_layout_horz::add_layout(Bcl_layout* layout, bool is_flex_width, bool is_flex_height)
{
	Layout_horz_info info;
	info.layout = layout;
	info.is_flex_width = is_flex_width;
	info.is_flex_height = is_flex_height;
	m_info.push_back(info);
	if (is_flex_width) m_flex_num++;
	info.layout = NULL;
	info.is_flex_width = false;
	info.is_flex_height = false;
	m_info.push_back(info);
	POINT pt = layout->get_position();
	if (pt.x < m_pt.x) m_pt.x = pt.x;
	if (pt.y < m_pt.y) m_pt.y = pt.y;
	pt.x += layout->get_width();
	int width = pt.x - m_pt.x;
	if (width > m_width) m_width = width;
	pt.y += layout->get_height();
	int height = pt.y - m_pt.y;
	if (height > m_height) m_height = height;
}
