//==============================================================
//
// Bcl_printer
//
//==============================================================
#include "bcl_printer.h"
#include "bcl_font.h"
#include "bcl_wnd.h"
#include "bcl_bmp.h"
#include "bcl_citr.h"

#define PAI 3.1415926

/// RXgN^
Bcl_printer::Bcl_printer() : m_hdc(NULL), m_color(RGB(0,0,0)), m_text_color(RGB(0,0,0)),
							 m_brush(BRUSH_TRANSPARENT), m_pen(PEN_SOLID), m_paper_size(PAPER_A4_VERT),
							 m_width(2892), m_height(4125) ,m_is_preview(true)
{
	DWORD need;
	DWORD rtn;
	EnumPrinters(PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS, NULL, 5, NULL, 0, &need, &rtn);
	if (need > 0)
	{
		PRINTER_INFO_5 *prninfo = (PRINTER_INFO_5 *)new char[need];
		EnumPrinters(PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS, NULL, 5, (LPBYTE)prninfo, need, &need, &rtn);
		for (DWORD i = 0; i < rtn; i++)
		{
			Prn_info prn_info;
			prn_info.name = prninfo[i].pPrinterName;
			prn_info.port_name = prninfo[i].pPortName;
			m_prn_info.push_back(prn_info);
		}
		delete[] prninfo;
	}
}

/// fXgN^
Bcl_printer::~Bcl_printer()
{
}

/// ftHgv^̖̎擾
/// @param parent eEBhE
/// @return v^
Bcl_str Bcl_printer::get_default_printer(Bcl_wnd* parent) const
{
	PAGESETUPDLG psd;
	memset(&psd, 0, sizeof(PAGESETUPDLG));
	psd.lStructSize = sizeof(PAGESETUPDLG);
	psd.hwndOwner = parent->get_handle();
	psd.Flags = PSD_RETURNDEFAULT;

	PageSetupDlg(&psd);
	//foCX̎擾
	DEVNAMES *pdev = (DEVNAMES *)GlobalLock(psd.hDevNames);
	Bcl_str rtn;
	if (pdev == NULL)
	{
		return rtn;
	}
	rtn = &((char *)pdev)[pdev->wDeviceOffset];
	GlobalUnlock(psd.hDevNames);
	GlobalFree(psd.hDevNames);
	return rtn;
}

/// v^[擾
/// @return v^[
int Bcl_printer::get_printer_num() const
{
	return m_prn_info.size();
}

/// v^[̎擾
/// @param idx CfbNX
/// @return v^[
Bcl_str Bcl_printer::get_printer_name(int idx) const
{
	return m_prn_info[idx].name;
}

/// |[g̎擾
/// @param idx CfbNX
/// @return |[g
Bcl_str Bcl_printer::get_port_name(int idx) const
{
	return m_prn_info[idx].port_name;
}

/// y[p[TCYݒ
/// @param size y[p[TCY
void Bcl_printer::set_paper_size(PAPER_SIZE size)
{
	m_paper_size = size;
	if (m_is_preview)
	{
		switch (m_paper_size)
		{
			case PAPER_A4_HORZ:
			{
				m_width = 4125;
				m_height = 2892;
				break;
			}
			case PAPER_A4_VERT:
			{
				m_width = 2892;
				m_height = 4125;
				break;
			}
			case PAPER_B5_HORZ:
			{
				m_width = 3559;
				m_height = 2496;
				break;
			}
			case PAPER_B5_VERT:
			{
				m_width = 2496;
				m_height = 3559;
				break;
			}
		}
	}
}

/// hLg̐ݒ
/// @param name hLg
void Bcl_printer::set_document_name(const Bcl_str& name)
{
	m_name = name;
}

/// foCXReLXg𐶐܂
/// @param idx CfbNX
void Bcl_printer::create_dc(int idx)
{
	HANDLE hPrint;
	DWORD dwNeeded;
	DEVMODE *pprn_info;
	PRINTER_DEFAULTS pdef;

	memset(&pdef, 0, sizeof(PRINTER_DEFAULTS));
	pdef.DesiredAccess = PRINTER_ALL_ACCESS;
	OpenPrinter(m_prn_info[idx].name.to_char(), &hPrint, &pdef);
	dwNeeded = DocumentProperties(NULL, hPrint, m_prn_info[idx].name.to_char(), NULL, NULL, 0);
	pprn_info = (DEVMODE *)new char[dwNeeded];
	DocumentProperties(NULL, hPrint, m_prn_info[idx].name.to_char(), pprn_info, NULL, DM_OUT_BUFFER);
	pprn_info->dmFields = DM_PAPERSIZE | DM_ORIENTATION;
	if ((m_paper_size == PAPER_A4_HORZ) || (m_paper_size == PAPER_A4_VERT))
	{
		pprn_info->dmPaperSize = DMPAPER_A4;
	}
	else
	{
		pprn_info->dmPaperSize = DMPAPER_B5;
	}
	if ((m_paper_size == PAPER_A4_VERT) || (m_paper_size == PAPER_B5_VERT))
	{
		pprn_info->dmOrientation = DMORIENT_PORTRAIT;
	}
	else
	{
		pprn_info->dmOrientation = DMORIENT_LANDSCAPE;
	}
	DocumentProperties(NULL, hPrint, m_prn_info[idx].name.to_char(), pprn_info, pprn_info, DM_OUT_BUFFER | DM_IN_BUFFER);
	m_hdc = CreateDC(NULL, m_prn_info[idx].name.to_char(), NULL, pprn_info);
	ClosePrinter(hPrint);
	delete[] pprn_info;
	m_width = GetDeviceCaps(m_hdc, HORZRES);
	m_height = GetDeviceCaps(m_hdc, VERTRES);
	m_is_preview = false;
}

/// foCXReLXgj
void Bcl_printer::delete_dc()
{
	DeleteDC(m_hdc);
	m_hdc = NULL;
	m_is_preview = true;
	set_paper_size(m_paper_size);
}

/// X^[g
void Bcl_printer::print_start()
{
	m_docinfo.cbSize = sizeof(DOCINFO);
	m_docinfo.lpszDocName = m_name.to_char();
	m_docinfo.lpszOutput = NULL;
	m_color = BLACK;
	m_text_color = BLACK;
	m_brush = BRUSH_TRANSPARENT;
	m_font = Bcl_font();
	StartDoc(m_hdc, &m_docinfo);
	StartPage(m_hdc);
}

/// I
void Bcl_printer::print_end()
{
	EndPage(m_hdc);
	EndDoc(m_hdc);
}

/// vr[Jn
/// @param parent eEBhE
void Bcl_printer::preview_start(Bcl_wnd* parent)
{
	//    TCY         BMPTCY    ͈
	// A4 297mm ~ 210mm 4158 ~ 2940 4125 ~ 2892
	// B5 257mm ~ 182mm 3598 ~ 2548 3559 ~ 2496
	switch (m_paper_size)
	{
		case PAPER_A4_HORZ:
		{
			m_mergin_up = 17;
			m_mergin_down = 16;
			m_mergin_left = 24;
			m_mergin_right = 24;
			break;
		}
		case PAPER_A4_VERT:
		{
			m_mergin_up = 24;
			m_mergin_down = 24;
			m_mergin_left = 17;
			m_mergin_right = 16;
			break;
		}
		case PAPER_B5_HORZ:
		{
			m_mergin_up = 20;
			m_mergin_down = 19;
			m_mergin_left = 26;
			m_mergin_right = 26;
			break;
		}
		case PAPER_B5_VERT:
		{
			m_mergin_up = 26;
			m_mergin_down = 26;
			m_mergin_left = 20;
			m_mergin_right = 19;
			break;
		}
	}
	m_color = BLACK;
	m_text_color = BLACK;
	m_brush = BRUSH_TRANSPARENT;
	m_preview_cvs.set_color(BLACK);
	m_preview_cvs.set_canvas(parent, (m_width + m_mergin_left + m_mergin_right) * 2 / 5, (m_height + m_mergin_up + m_mergin_down) * 2 / 5);
	m_preview_cvs.set_brush_type(BRUSH_TRANSPARENT);
	m_preview_cvs.set_bg_color(WHITE);
	m_font = Bcl_font();
	m_preview_cvs.set_font(m_font);
	m_preview_cvs.clear();
	m_is_preview = true;
}

/// vr[I
void Bcl_printer::preview_end()
{
	m_preview_cvs.set_color(WHITE);
	m_preview_cvs.set_brush_type(BRUSH_SOLID);
	m_preview_cvs.box(0, 0, (m_width + m_mergin_left + m_mergin_right) * 2 / 5 - 1, m_mergin_up * 2 / 5 - 1);
	m_preview_cvs.box(0, 0, m_mergin_left * 2 / 5 - 1, (m_height + m_mergin_up + m_mergin_down) * 2 / 5 - 1);
	m_preview_cvs.box(0, (m_height + m_mergin_up) * 2 / 5, (m_width + m_mergin_left + m_mergin_right) * 2 / 5 - 1, (m_height + m_mergin_up + m_mergin_down) * 2 / 5 - 1);
	m_preview_cvs.box((m_mergin_left + m_width) * 2 / 5, 0, (m_width + m_mergin_left + m_mergin_right) * 2 / 5 - 1, (m_height + m_mergin_up + m_mergin_down) * 2 / 5 - 1);
	m_preview_cvs.set_color(m_color);
}

/// vr[ʎ擾
/// @return vr[
Bcl_bmp* Bcl_printer::get_preview_bmp() const
{
	return m_preview_cvs.get_bmp(0, 0, (m_width + m_mergin_left + m_mergin_right) * 2 / 5, (m_height + m_mergin_up + m_mergin_down) * 2 / 5);
}

/// y[W
void Bcl_printer::next_page()
{
	if (m_is_preview)
	{
		m_preview_cvs.clear();
	}
	else
	{
		EndPage(m_hdc);
		StartPage(m_hdc);
	}
}

/// y[p[TCY擾
/// @return y[p[TCY
PAPER_SIZE Bcl_printer::get_paper_size() const
{
	return m_paper_size;
}

/// p擾
/// @return p
int Bcl_printer::get_width() const
{
	return m_width;
}

/// p擾
/// @return p
int Bcl_printer::get_height() const
{
	return m_height;
}

/// Fݒ
/// @param col F
void Bcl_printer::set_color(COLORREF col)
{
	m_color = col;
	if (m_is_preview)
	{
		m_preview_cvs.set_color(col);
	}
}

/// eLXgFݒ
/// @param col eLXgF
void Bcl_printer::set_text_color(COLORREF col)
{
	if (m_is_preview)
	{
		m_preview_cvs.set_text_color(col);
	}
	else
	{
		m_text_color = col;
	}
}

/// uV^Cvݒ
/// @param brush uV^Cv
void Bcl_printer::set_brush_type(BRUSH_TYPE brush)
{
	if (m_is_preview)
	{
		m_preview_cvs.set_brush_type(brush);
	}
	else
	{
		m_brush = brush;
	}
}

/// y^Cvݒ
/// @param pen y^Cv
void Bcl_printer::set_pen_type(PEN_TYPE pen)
{
	if (m_is_preview)
	{
		m_preview_cvs.set_pen_type(pen);
	}
	else
	{
		m_pen = pen;
	}
}

/// _`
/// @param x W
/// @param y W
void Bcl_printer::dot(int x, int y)
{
	if (m_is_preview)
	{
		m_preview_cvs.dot((x + m_mergin_left) * 2 / 5, (y + m_mergin_up) * 2 / 5);
	}
	else
	{
		SetPixel(m_hdc, x, y, m_color);
	}
}

/// `
/// @param sx n_W
/// @param sy n_W
/// @param ex I_W
/// @param ey I_W
void Bcl_printer::line(int sx, int sy, int ex, int ey)
{
	if (m_is_preview)
	{
		m_preview_cvs.line((sx + m_mergin_left) * 2 / 5, (sy + m_mergin_up) * 2 / 5, (ex + m_mergin_left) * 2 / 5, (ey + m_mergin_up) * 2 / 5);
	}
	else
	{
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		MoveToEx(m_hdc, sx, sy, NULL);
		LineTo(m_hdc, ex, ey);
		SelectObject(m_hdc, old_pen);
		DeleteObject(pen);
		dot(ex, ey);
	}
}

/// `
/// @param sx ゘W
/// @param sy ゙W
/// @param ex EW
/// @param ey EW
void Bcl_printer::box(int sx, int sy, int ex, int ey)
{
	if (m_is_preview)
	{
		m_preview_cvs.box((sx + m_mergin_left) * 2 / 5, (sy + m_mergin_up) * 2 / 5, (ex + m_mergin_left) * 2 / 5, (ey + m_mergin_up) * 2 / 5);
	}
	else
	{
		if (sx > ex)
		{
			int tmp = sx;
			sx = ex;
			ex = tmp;
		}
		if (sy > ey)
		{
			int tmp = sy;
			sy = ey;
			ey = tmp;
		}
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		HBRUSH brush = create_brush(m_color, m_brush);
		HBRUSH old_brush = (HBRUSH)SelectObject(m_hdc, brush);
		Rectangle(m_hdc, sx, sy, ex + 1, ey + 1);
		SelectObject(m_hdc, old_pen);
		SelectObject(m_hdc, old_brush);
		DeleteObject(pen);
		DeleteObject(brush);			//NULL_BRUSH̏ꍇł͖
	}
}

/// ~`
/// @param x SW
/// @param y SW
/// @param r a
void Bcl_printer::circle(int x, int y, int r)
{
	if (m_is_preview)
	{
		m_preview_cvs.circle((x + m_mergin_left) * 2 / 5, (y + m_mergin_up) * 2 / 5, r * 2 / 5);
	}
	else
	{
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		HBRUSH brush = create_brush(m_color, m_brush);
		HBRUSH old_brush = (HBRUSH)SelectObject(m_hdc, brush);
		Ellipse(m_hdc, x - r, y - r, x + r, y + r);
		SelectObject(m_hdc, old_pen);
		SelectObject(m_hdc, old_brush);
		DeleteObject(pen);
		DeleteObject(brush);			//NULL_BRUSH̏ꍇł͖
	}
}

/// ȉ~`
/// @param sx Oڒ`̍゘W
/// @param sy Oڒ`゙̍W
/// @param ex Oڒ`̉EW
/// @param ey Oڒ`̉EW
void Bcl_printer::ellipse(int sx, int sy, int ex, int ey)
{
	if (m_is_preview)
	{
		m_preview_cvs.ellipse((sx + m_mergin_left) * 2 / 5, (sy + m_mergin_up) * 2 / 5, (ex + m_mergin_left) * 2 / 5, (ey + m_mergin_up) * 2 / 5);
	}
	else
	{
		if (sx > ex)
		{
			int tmp = sx;
			sx = ex;
			ex = tmp;
		}
		if (sy > ey)
		{
			int tmp = sy;
			sy = ey;
			ey = tmp;
		}
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		HBRUSH brush = create_brush(m_color, m_brush);
		HBRUSH old_brush = (HBRUSH)SelectObject(m_hdc, brush);
		Ellipse(m_hdc, sx, sy, ex + 1, ey + 1);
		SelectObject(m_hdc, old_pen);
		SelectObject(m_hdc, old_brush);
		DeleteObject(pen);
		DeleteObject(brush);			//NULL_BRUSH̏ꍇł͖
	}
}

/// ~ʕ`
/// @param sx ȉ~̊Oڒ`̍゘W
/// @param sy ȉ~̊Oڒ`゙̍W
/// @param ex ȉ~̊Oڒ`̉EW
/// @param ey ȉ~̊Oڒ`̉EW
/// @param srx ȉ~̌ʂ̎n_`˒̒[_̂W
/// @param sry ȉ~̌ʂ̎n_`˒̒[_̂W
/// @param erx ȉ~̌ʂ̏I_`˒̒[_̂W
/// @param ery ȉ~̌ʂ̏I_`˒̒[_̂W
void Bcl_printer::arc(int sx, int sy, int ex, int ey, int srx, int sry, int erx, int ery)
{
	if (m_is_preview)
	{
		m_preview_cvs.arc((sx + m_mergin_left) * 2 / 5, (sy + m_mergin_up) * 2 / 5, (ex + m_mergin_left) * 2 / 5, (ey + m_mergin_up) * 2 / 5,
						  (srx + m_mergin_left) * 2 / 5, (sry + m_mergin_up) * 2 / 5, (erx + m_mergin_left) * 2 / 5, (ery + m_mergin_up) * 2 / 5);
	}
	else
	{
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		Arc(m_hdc, sx, sy, ex, ey, srx, sry, erx, ery);
		SelectObject(m_hdc, old_pen);
		DeleteObject(pen);
	}
}

/// `
/// @param sx ȉ~̊Oڒ`̍゘W
/// @param sy ȉ~̊Oڒ`゙̍W
/// @param ex ȉ~̊Oڒ`̉EW
/// @param ey ȉ~̊Oڒ`̉EW
/// @param srx ȉ~̌ʂ̎n_`˒̒[_̂W
/// @param sry ȉ~̌ʂ̎n_`˒̒[_̂W
/// @param erx ȉ~̌ʂ̏I_`˒̒[_̂W
/// @param ery ȉ~̌ʂ̏I_`˒̒[_̂W
void Bcl_printer::chord(int sx, int sy, int ex, int ey, int srx, int sry, int erx, int ery)
{
	if (m_is_preview)
	{
		m_preview_cvs.chord((sx + m_mergin_left) * 2 / 5, (sy + m_mergin_up) * 2 / 5, (ex + m_mergin_left) * 2 / 5, (ey + m_mergin_up) * 2 / 5,
							(srx + m_mergin_left) * 2 / 5, (sry + m_mergin_up) * 2 / 5, (erx + m_mergin_left) * 2 / 5, (ery + m_mergin_up) * 2 / 5);
	}
	else
	{
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		HBRUSH brush = create_brush(m_color, m_brush);
		HBRUSH old_brush = (HBRUSH)SelectObject(m_hdc, brush);
		Chord(m_hdc, sx, sy, ex, ey, srx, sry, erx, ery);
		SelectObject(m_hdc, old_pen);
		SelectObject(m_hdc, old_brush);
		DeleteObject(pen);
		DeleteObject(brush);			//NULL_BRUSH̏ꍇł͖
	}
}

/// p``
/// @param pts _̗
void Bcl_printer::polygon(const std::vector<POINT>& pts)
{
	if (m_is_preview)
	{
		std::vector<POINT> mergin_pts;
		Bcl_citr< std::vector<POINT> > itr(pts);
		while (itr.has_next())
		{
			POINT pt = itr.next();
			pt.x += m_mergin_left;
			pt.y += m_mergin_up;
			pt.x = pt.x * 2 / 5;
			pt.y = pt.y * 2 / 5;
			mergin_pts.push_back(pt);
		}
		m_preview_cvs.polygon(mergin_pts);
	}
	else
	{
		HPEN pen = CreatePen(get_pen_type(), 0, m_color);
		HPEN old_pen = (HPEN)SelectObject(m_hdc, pen);
		HBRUSH brush = create_brush(m_color, m_brush);
		HBRUSH old_brush = (HBRUSH)SelectObject(m_hdc, brush);
		POINT* lp_point = new POINT[pts.size()];
		Bcl_citr< std::vector<POINT> > itr(pts);
		POINT min;
		POINT max;
		min.x = 0x7fffffff;
		min.y = 0x7fffffff;
		max.x = 0x80000000;
		max.y = 0x80000000;
		while (itr.has_next())
		{
			int i = itr.get_count();
			lp_point[i] = itr.next();
			if (min.x > lp_point[i].x) min.x = lp_point[i].x;
			if (min.y > lp_point[i].y) min.y = lp_point[i].y;
			if (max.x < lp_point[i].x) max.x = lp_point[i].x;
			if (max.y < lp_point[i].y) max.y = lp_point[i].y;
		}
		Polygon(m_hdc, lp_point, pts.size());
		delete[] lp_point;
		SelectObject(m_hdc, old_pen);
		SelectObject(m_hdc, old_brush);
		DeleteObject(pen);
		DeleteObject(brush);			//NULL_BRUSH̏ꍇł͖
	}
}

/// rbg}bv`
/// @param bmp rbg}bv
/// @param x W
/// @param y W
/// @param ro X^Iy[VR[h
void Bcl_printer::put_bmp(const Bcl_bmp& bmp, int x, int y, DWORD ro)
{
	if (m_is_preview)
	{
		m_preview_cvs.put_bmp(bmp, (x + m_mergin_left) * 2 / 5, (y + m_mergin_up) * 2 / 5, ro);
	}
	else
	{
		fit_bmp(bmp, x, y, bmp.get_width(), bmp.get_height(), ro);
	}
}

/// rbg}bv`igEkj
/// @param bmp rbg}bv
/// @param x W
/// @param y W
/// @param w 
/// @param h 
/// @param ro X^Iy[VR[h
/// @param mode Xgb`O[h
void Bcl_printer::fit_bmp(const Bcl_bmp& bmp, int x, int y, int w, int h, DWORD ro, int mode)
{
	if (m_is_preview)
	{
		m_preview_cvs.fit_bmp(bmp, (x + m_mergin_left) * 2 / 5, (y + m_mergin_up) * 2 / 5, w * 2 / 5, h * 2 / 5, ro, mode);
	}
	else
	{
		if (bmp.get_handle() == NULL)
		{
			return;			// leave function
		}

		LPBYTE bits;			// [ւ̃|C^[
		DWORD bmp_size;			//rbg}bvTCYvZp
		int rtn;			//GetDIBits֐̐tO
		HDC dc, mem;			//fXNgbvDCƃRp`rbg}bvDC
		PBITMAPINFO info;		//ݗp
		PBITMAPFILEHEADER fheder;	//ݗp

		WORD col_bit = bmp.get_color_bit();
		//PBITMAPINFO inföׂ̃[m
		if (col_bit <= 8)
		{
			info = (PBITMAPINFO)LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * (1 << col_bit));
		}
		else
		{
			//24bittH[}bg͖ܘ_16, 32bitBI_RGB̏ꍇɂRGBQUADȂ
			info = (PBITMAPINFO)LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER));
		}
		if(!info)
		{
			//[mێs
			MessageBox(NULL, "infõ[J[mۂɎs܂",
						"G[!", MB_OK | MB_ICONSTOP);
			return;
		}
		//BITMAPINFȌ
		info->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
		info->bmiHeader.biWidth		= bmp.get_width();
		info->bmiHeader.biHeight	= bmp.get_height();
		info->bmiHeader.biPlanes	= 1;
		info->bmiHeader.biBitCount	= col_bit;
		info->bmiHeader.biCompression	= BI_RGB;		//0x0
		info->bmiHeader.biXPelsPerMeter	= 0;
		info->bmiHeader.biYPelsPerMeter	= 0;
		info->bmiHeader.biClrUsed		= 0;
		info->bmiHeader.biClrImportant	= 0;
		//rbg}bvf[^TCYZo
		bmp_size = ((col_bit * info->bmiHeader.biWidth + 31) / 32) * 4
					* abs(info->bmiHeader.biHeight);
		info->bmiHeader.biSizeImage	= bmp_size;
		//BITMAPFILEHEADEȐ
		fheder = (PBITMAPFILEHEADER)LocalAlloc(LPTR,
				sizeof(BITMAPFILEHEADER));
		if(!fheder)
		{
			//[mێs
			MessageBox(NULL, "fheder̃[J[mۂɎs܂",
						"G[!", MB_OK | MB_ICONSTOP);
			LocalFree(info);
			return;
		}
		fheder->bfType = 0x4D42;
		fheder->bfReserved1 = fheder->bfReserved2 = 0;
		if(col_bit <= 8)	//BITMAPFILEHEADER + BITMAPINFOHEADER + RGBQUAD
			fheder->bfOffBits = 54 + (4 * (1 << col_bit));
		else			//BITMAPFILEHEADER + BITMAPINFOHEADER
			fheder->bfOffBits = 54;
		fheder->bfSize = bmp_size + fheder->bfOffBits;
		bits = (LPBYTE)GlobalAlloc(GMEM_FIXED, bmp_size);
		if(!bits)
		{
			MessageBox(NULL, "O[o[mۂł܂ł",
						"G[", MB_OK | MB_ICONSTOP);
			LocalFree(info);
			LocalFree(fheder);
			return;
		}
		//ŐFύX
		dc = GetDC(0);					//fXNgbṽfoCXReLXg
		mem = CreateCompatibleDC(dc);	//̃Rp`DC
		SelectObject(mem, bmp.get_handle());		//rbg}bvǂ܂
		//rbgzGetDIBits֐ŃO[o[Ɏo
		rtn = GetDIBits(mem, bmp.get_handle(), 0, bmp.get_height(),
								bits, info, DIB_RGB_COLORS);
		DeleteDC(mem) ;				//Rp`DC폜
		ReleaseDC(0, dc);				//fXNgbṽfoCXReLXgJ
		if(!rtn) {	//FύXs
			MessageBox(NULL, "FύXɎs܂",
						"G[", MB_OK | MB_ICONSTOP);
			LocalFree(info);
			LocalFree(fheder);
			GlobalFree((HGLOBAL)bits);
			return;
		}
		SetStretchBltMode(m_hdc, mode);
		StretchDIBits(m_hdc, x, y, w, h, 0, 0, bmp.get_width(), bmp.get_height(), bits, info, DIB_RGB_COLORS, ro);
		//[
		LocalFree(info);
		LocalFree(fheder);
		GlobalFree((HGLOBAL)bits);
	}
}

/// eLXg`
/// @param x W
/// @param y W
/// @param str eLXg
/// @param w `敝
/// @param fmt eLXg`IvV
void Bcl_printer::print_text(int x, int y, const Bcl_str& str, int w, UINT fmt)
{
	if (m_is_preview)
	{
		m_preview_cvs.print_text((x + m_mergin_left) * 2 / 5, (y + m_mergin_up) * 2 / 5, str, w * 2 / 5, fmt);
	}
	else
	{
		RECT rec;
		SetTextColor(m_hdc, m_text_color);
		HFONT old_font;
		if (m_font.is_valid())
		{
			old_font = (HFONT)SelectObject(m_hdc, m_font.get_handle());
		}
		rec.right = get_width();
		rec.bottom = get_height();
		rec.left = x;
		rec.top = y;
		if (w > 0)	rec.right = x + w;
		RECT chk_rec = rec;
		rec.bottom = y + DrawText(m_hdc, str.to_char(), -1, &chk_rec, fmt | DT_CALCRECT);
		DrawText(m_hdc, str.to_char(), -1, &rec, fmt);
		if (m_font.is_valid())
		{
			SelectObject(m_hdc, old_font);
		}
	}
}

/// eLXg`icj
/// @param x W
/// @param y W
/// @param str eLXg
/// @param size ő`敶
void Bcl_printer::print_htext(int x, int y, const Bcl_str& str, int size, UINT fmt)
{
	if (m_is_preview)
	{
		m_preview_cvs.print_htext((x + m_mergin_left) * 2 / 5, (y + m_mergin_up) * 2 / 5, str, size, fmt);
	}
	else
	{
		char moji[3];
		int i, j, len;

		i = 0;
		j = 0;
		len = strlen(str.to_char());
		Bcl_font font_hold = m_font;
		m_font.set_font_effect(m_font.get_bold(), m_font.is_italic(), false, false);
		double angle = 0;
		if (m_font.is_valid())
		{
			angle = PAI / 180 * (m_font.get_angle() / 10);
		}
		while ((i < len) && (j < size))
		{
			if (((0x81 <= (unsigned char)str[i]) && ((unsigned char)str[i] <= 0x9F)) ||
				((0xE0 <= (unsigned char)str[i])))
			{
				moji[0] = str[i];
				moji[1] = str[i + 1];
				moji[2] = 0;
				i += 2;
				int height = j * m_font.get_height();
				if ((strcmp(moji, "i") == 0) || (strcmp(moji, "j") == 0) || (strcmp(moji, "[") == 0))
				{
					Bcl_font font = m_font;
					m_font.set_angle((m_font.get_angle() + 2700) % 3600);
				 	int width = m_font.get_height();
				 	print_text(x + sin(angle) * height + cos(angle) * width, y + cos(angle) * height - sin(angle) * width, moji, 0, fmt);
				 	m_font = font;
				}
				else
				{
				 	print_text(x + sin(angle) * height, y + cos(angle) * height, moji, 0, fmt);
				}
			}
			else
			{
				moji[0] = str[i];
				moji[1] = 0;
				i++;
				if ((strcmp(moji, "(") == 0) || (strcmp(moji, ")") == 0) || (strcmp(moji, "-") == 0))
				{
					Bcl_font font = m_font;
					m_font.set_angle((m_font.get_angle() + 2700) % 3600);
					int height = j * m_font.get_height() + m_font.get_height() / 4;
					int width = m_font.get_height();
					print_text(x + sin(angle) * height + cos(angle) * width, y + cos(angle) * height - sin(angle) * width, moji, 0, fmt);
				 	m_font = font;
				}
				else
				{
					int height = j * m_font.get_height();
					int width = m_font.get_height() / 4;
					print_text(x + sin(angle) * height + cos(angle) * width, y + cos(angle) * height - sin(angle) * width, moji, 0, fmt);
				}
			}
			j++;
		}
		COLORREF color = m_color;
		m_color = m_text_color;
		int height = j * m_font.get_height() - 1;
		int width = m_font.get_height() / 2;
		if (font_hold.is_underline())
		{
			line(x, y, x + sin(angle) * height, y + cos(angle) * height);
		}
		if (font_hold.is_strikeout())
		{
			line(x + cos(angle) * width, y - sin(angle) * width, x + cos(angle) * width + sin(angle) * height, y - sin(angle) * width + cos(angle) * height);
		}
		m_color = color;
		m_font = font_hold;
	}
}

/// tHgݒ
/// @param font tHg
void Bcl_printer::set_font(const Bcl_font& font)
{
	m_font = font;
	if (m_is_preview)
	{
		Bcl_font preview_font = font;
		int height = font.get_height() * 2 / 5;
		if (height <= 0) height = 1;
		preview_font.set_height(height);
		m_preview_cvs.set_font(preview_font);
	}
}

/// tHgTCYݒ
/// @param size tHgTCY
void Bcl_printer::set_font_size(int size)
{
	if (m_is_preview)
	{
		set_font_height(size * 5);
	}
	else
	{
		m_font.set_font(MulDiv(size, GetDeviceCaps(m_hdc, LOGPIXELSY), -72), m_font.get_font_name(), m_font.get_angle());
	}
}

/// tHgݒ
/// @param height tHg
void Bcl_printer::set_font_height(int height)
{
	m_font.set_height(height);
	if (m_is_preview)
	{
		height = height * 2 / 5;
		if (height <= 0) height = 1;
		m_preview_cvs.set_font_height(height);
	}
}

/// tHg̊pxݒ
/// @param angle px
void Bcl_printer::set_font_angle(int angle)
{
	m_font.set_angle(angle);
	if (m_is_preview) m_preview_cvs.set_font_angle(angle);
}

/// tHǧʂݒ肵܂
/// @param bold 
/// @param italic Α
/// @param underline 
/// @param strikeout 
void Bcl_printer::set_font_effect(LONG bold, bool italic, bool underline, bool strikeout)
{
	m_font.set_font_effect(bold, italic, underline, strikeout);
	if (m_is_preview) m_preview_cvs.set_font_effect(bold, italic, underline, strikeout);
}

/// tHg擾
/// @return tHg
const Bcl_font& Bcl_printer::get_font() const
{
	return m_font;
}

/// tHg擾
/// @param size tHgTCY
/// @return tHg
int Bcl_printer::get_font_height(int size) const
{
	if (m_is_preview)
	{
		return size * 5;
	}
	return MulDiv(size, GetDeviceCaps(m_hdc, LOGPIXELSY), 72);
}

/// uV
/// @param col F
/// @param type uV^Cv
/// @return uV
HBRUSH Bcl_printer::create_brush(COLORREF col, BRUSH_TYPE type)
{
	HBRUSH brush;
	switch (type)
	{
	case BRUSH_SOLID:
		brush = CreateSolidBrush(col);
		break;
	case BRUSH_HHATCH:
		brush = CreateHatchBrush(HS_HORIZONTAL, col);
		break;
	case BRUSH_VHATCH:
		brush = CreateHatchBrush(HS_VERTICAL, col);
		break;
	case BRUSH_HVHATCH:
		brush = CreateHatchBrush(HS_CROSS, col);
		break;
	case BRUSH_RHATCH:
		brush = CreateHatchBrush(HS_BDIAGONAL, col);
		break;
	case BRUSH_LHATCH:
		brush = CreateHatchBrush(HS_FDIAGONAL, col);
		break;
	case BRUSH_RLHATCH:
		brush = CreateHatchBrush(HS_DIAGCROSS, col);
		break;
	case BRUSH_TRANSPARENT:
	default:
		brush = (HBRUSH)GetStockObject(NULL_BRUSH);
		break;
	}
	return brush;
}

/// y^Cv擾
/// @return y^Cv
int Bcl_printer::get_pen_type()
{
	int rtn;
	switch (m_pen)
	{
		case PEN_SOLID:
			rtn = PS_SOLID;
			break;
		case PEN_DASH:
			rtn = PS_DASH;
			break;
		case PEN_DOT:
			rtn = PS_DOT;
			break;
		case PEN_DASHDOT:
			rtn = PS_DASHDOT;
			break;
		case PEN_DASHDOTDOT:
			rtn = PS_DASHDOTDOT;
			break;
		default:
			rtn = PS_SOLID;
			break;
	}
	return rtn;
}
